package com.persagy.proxy.adm.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.persagy.proxy.adm.request.AdmQueryCriteria;
import com.persagy.proxy.adm.request.QueryCriteria;
import com.persagy.proxy.adm.service.IAdmBaseService;
import com.persagy.proxy.adm.utils.AdmEntityTransferUtil;
import com.persagy.proxy.adm.utils.AdmQueryCriteriaHelper;
import com.persagy.proxy.common.client.DmpRwdClient;
import com.persagy.proxy.common.entity.DmpResult;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

/**
 * ADM 抽象实现类
 * @author Charlie Yu
 * @date 2021-08-16
 */
@Slf4j
public class AbstractAdmBaseServiceImpl<T> implements IAdmBaseService<T> {

    @Autowired
    private DmpRwdClient rwdClient;
    @Autowired
    private ObjectMapper objectMapper;

    /**
     * 查询
     * @param request
     * @return
     */
    @Override
    public DmpResult<JSONArray> doQuery(InstanceUrlParam context, AdmQueryCriteria request) {
        // 转换为数据中台查询条件
        QueryCriteria dmpRequest = AdmQueryCriteriaHelper.toDmpCriteria(request);
        // 转换参数
        JSONObject para = null;
        try {
            String paraStr = objectMapper.writeValueAsString(dmpRequest);
            para = JSONObject.parseObject(paraStr);
        } catch (JsonProcessingException e) {
            log.error(e.getMessage(), e);
            return null;
        }
        // 调用中台查询
        return rwdClient.queryObject(context, para);
    }

    /**
     * 拷贝属性
     * @param dmpData 数据中台对象
     * @param srcProp 数据中台属性名
     * @param desProp 对应的数据中心属性名
     */
    private void copyProperty(JSONObject dmpData, String srcProp, String desProp) {
        Object value = dmpData.get(srcProp);
        if(value == null) {
            return;
        }
        dmpData.put(desProp, value);
        dmpData.remove(srcProp);
    }

    /**
     * 新增
     * @param context
     * @param clazz
     * @param voList
     * @return
     */
    @Override
    public List<T> doInsert(InstanceUrlParam context, Class<T> clazz, List<T> voList) {
        if(CollUtil.isEmpty(voList)) {
            return null;
        }
        ArrayNode vos = AdmEntityTransferUtil.toDmpMultiEntity(voList);
        // 调用中台新增
        DmpResult<JSONArray> response = rwdClient.createObject(context, JSONArray.parseArray(vos.toString()));
        if(!DmpResult.SUCCESS.equals(response.getResult())) {
            throw new RuntimeException("调用中台查询接口出错：" + response.getMessage());
        }
        // 转换为结果
        return AdmEntityTransferUtil.toAdmMultiEntity(response.getData(), clazz);
    }

    /**
     * 修改
     * @param context
     * @param clazz
     * @param voList
     * @return
     */
    @Override
    public List<T> doUpdate(InstanceUrlParam context, Class<T> clazz, List<T> voList) {
        if(CollUtil.isEmpty(voList)) {
            return null;
        }
        ArrayNode vos = AdmEntityTransferUtil.toDmpMultiEntity(voList);
        // 调用中台修改
        DmpResult<cn.hutool.json.JSONArray> response = rwdClient.updateObject(context, JSONArray.parseArray(vos.toString()));
        if(!DmpResult.SUCCESS.equals(response.getResult())) {
            throw new RuntimeException("调用中台查询接口出错：" + response.getMessage());
        }
        // 转换为结果
        return AdmEntityTransferUtil.toAdmMultiEntity(response.getData(), clazz);
    }

    /**
     * 删除
     * @param context
     * @param voList
     * @return
     */
    @Override
    public void doDelete(InstanceUrlParam context, List<T> voList) {
        if(CollUtil.isEmpty(voList)) {
            return;
        }
        List<String> idList = CollUtil.getFieldValues(voList, "id", String.class);
        // 调用中台修改
        DmpResult response = rwdClient.deleteObject(context, idList);
        if(!DmpResult.SUCCESS.equals(response.getResult())) {
            throw new RuntimeException("调用中台查询接口出错：" + response.getMessage());
        }
    }
}
